package org.nst.pasientlink.authenticator;

/**
 * <p>Title: AuthenticatorStorage</p>
 * <p>Description: This class handles persistency in the authenticator class</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Norwegian Centre for Telemedicine </p>
 * @author Thomas Strandens, Per Atle Bakkevoll
 * @version 1.0
 *
 */

import java.util.*;
import java.sql.*;
import java.io.*;
import com.javaexchange.dbConnectionBroker.*;
import org.nst.pasientlink.authenticator.*;
import org.apache.log4j.Logger;

public class AuthenticatorStorage {

   private static DbConnectionBroker myDbBroker;
   private Logger logger;

   /**
    * Constructor. Initializes database connection pool from configuration,
    * and sets up logger.
    *
    * @param myProperties Properties containing configuration properties
    * @see com.javaexchange.dbConnectionBroker.DBConnectionBroker
    */

   public AuthenticatorStorage(Properties myProperties) {

      try {
         logger = Utility.getLogger();
         myDbBroker = new DbConnectionBroker(myProperties.getProperty("db_driver"),
                                             myProperties.getProperty("db_url"),
                                             myProperties.getProperty("db_username"),
                                             myProperties.getProperty("db_password"),
                                             new Integer(myProperties.getProperty("db_min_connections")).intValue(),
                                             new Integer(myProperties.getProperty("db_max_connections")).intValue(),
                                             myProperties.getProperty("db_logfile"),
                                             new Double(myProperties.getProperty("db_max_connection_time")).doubleValue());
	   }
      catch (Exception e)  {
         logger.fatal("Could not create database connection or connection pool. Check properties and database setup: " + e.getMessage());
	   }
   }

   /**
    * createUser creates a user entry in storage from the specified input
    * parameters, given that a user with the same userid does not exist.
    *
    * @param myUserID String containing userID of the new user
    * @param myUserDescription String containing a description, typically name, of the new user
    * @param myPasswordHash byte array containing password hash for the new user
    * @param myGSMSubscriberID String containing the GSM phone number of the new user
    * @return a boolean value indicating whether the user was created (true) or not (false)
    *
    */

   public boolean createUser(String myUserID, String myUserDescription, byte[] myPasswordHash, String myGSMSubscriberID) {
      Connection myConnection = null;
      Statement myStatement = null;
      PreparedStatement myPreparedStatement =  null;
      ResultSet myResultSet = null;
		boolean retVal = false;

      if(myUserID != null &&
         myPasswordHash != null &&
         myGSMSubscriberID != null) {

         if(!this.userExists(myUserID)) {
            try {
               myConnection = getConnection();
               myPreparedStatement= myConnection.prepareStatement("INSERT INTO users " +
                                                                     "(user_id, " +
                                                                      "user_description, " +
                                                                      "password, " +
                                                                      "gsm_subscriber_id, " +
                                                                      "date_created ) " +
                                                                  "VALUES (?, ?, ?, ?, ?)");
               myPreparedStatement.setString(1, myUserID);
               if(myUserDescription == null) {
                  myPreparedStatement.setNull(2, Types.VARCHAR);
               }
               else {
                  myPreparedStatement.setString(2, myUserDescription);
               }
               myPreparedStatement.setBytes(3, myPasswordHash);
               myPreparedStatement.setString(4, myGSMSubscriberID);

               myPreparedStatement.setDate(5, new java.sql.Date(System.currentTimeMillis()));
               myPreparedStatement.execute();

               // commit the insert, and we're done
               myConnection.commit();
               logger.info("Created user with UserID '" + myUserID + "'");
               retVal = true;
            }
            catch (SQLException e) {
               logger.error("Unable to create new user with userID '" + myUserID + "': " + e.getMessage());
            }
            finally { // clean up
               try {
                  if(myStatement != null) {
                     myStatement.close();
                  }
               } catch(SQLException anotherException) {
                  logger.error(anotherException.getMessage());
               };
               try {
                  if(myPreparedStatement != null) {
                     myPreparedStatement.close();
                  }
               }
               catch(SQLException anotherException) {
                  logger.error(anotherException.getMessage());
               };
               // return connection to pool
               this.myDbBroker.freeConnection(myConnection);
            }
         }
      }
      return retVal;
   }

   /**
    * removeUser removes a specified userid from the database
    *
    * @param myUserID String containing userID of the soon-to-be-gone user
    * @return a boolean value indicating whether the user was removed (true) or not (false)
    *
    */

   public boolean removeUser(String myUserID) {
      Connection myConnection = null;
      Statement myStatement = null;
      PreparedStatement myPreparedStatement =  null;
      ResultSet myResultSet = null;
		boolean retVal = false;

      if(myUserID != null) {
         if(this.userExists(myUserID)) {
            try {
               myConnection = getConnection();
               myPreparedStatement= myConnection.prepareStatement("DELETE FROM users " +
                                                                  "WHERE user_id = ?");
               myPreparedStatement.setString(1, myUserID);
               myPreparedStatement.execute();

               // commit the delete, and we're done
               myConnection.commit();
               logger.info("Removed user with UserID '" + myUserID + "'");
               retVal = true;
            }
            catch (SQLException e) {
               logger.error("Unable to remove user with userID '" + myUserID + "': " + e.getMessage());
            }
            finally { // clean up
               try {
                  if(myStatement != null) {
                     myStatement.close();
                  }
               } catch(SQLException anotherException) {
                  logger.error("Unable to remove user: " + anotherException.getMessage());
               };
               try {
                  if(myPreparedStatement != null) {
                     myPreparedStatement.close();
                  }
               }
               catch(SQLException anotherException) {
                  logger.error("Unable to remove user: " + anotherException.getMessage());
               };
               // return connection to pool
               this.myDbBroker.freeConnection(myConnection);
            }
         }
         else {
            logger.warn("Unable to remove non-existent user");
         }
      }
      else {
         logger.warn("Unable to remove user with null UserID");
      }
      return retVal;
   }

   /**
    * changePassword updates the password property for a specified userid in the database
    *
    * @param myUserID String containing userID of the user
    * @param myNewPasswordHash byte array containing the password hash of the new password
    * @return a boolean value indicating whether the password was changed (true) or not (false)
    */

   public boolean changePassword(String myUserID,
                                 byte[] myNewPasswordHash) {
      Connection myConnection = null;
      Statement myStatement = null;
      PreparedStatement myPreparedStatement =  null;
      ResultSet myResultSet = null;
		boolean retVal = false;

      if(myUserID != null && myNewPasswordHash != null) {
         if(this.userExists(myUserID)) {
            try {
               myConnection = getConnection();
               myPreparedStatement= myConnection.prepareStatement("UPDATE users " +
                                                                  "SET password = ? " +
                                                                  "WHERE user_id = ?");
               myPreparedStatement.setBytes(1, myNewPasswordHash);
               myPreparedStatement.setString(2, myUserID);
               myPreparedStatement.execute();

               // commit the update, and we're done
               myConnection.commit();
               logger.info("Updated password for user with UserID '" + myUserID + "'");
               retVal = true;
            }
            catch (SQLException e) {
               logger.error("Unable to update password for user with userID '" + myUserID + "': " + e.getMessage());
            }
            finally { // clean up
               try {
                  if(myStatement != null) {
                     myStatement.close();
                  }
               } catch(SQLException anotherException) {
                  logger.error("Unable to update password: " + anotherException.getMessage());
               };
               try {
                  if(myPreparedStatement != null) {
                     myPreparedStatement.close();
                  }
               }
               catch(SQLException anotherException) {
                  logger.error("Unable to update password: " + anotherException.getMessage());
               };
               // return connection to pool
               this.myDbBroker.freeConnection(myConnection);
            }
         }
         else {
            logger.warn("Unable to change password for non-existent user");
         }
      }
      else {
         logger.warn("Unable to update password for user with null user id/null new password hash");
      }
      return retVal;
   }

   /**
    * getUserExists verifies the existence of a specified user in the database
    *
    * @param myUserID String containing a user_id
    * @return boolean indicating whether (true) or not (falsE) the specified user exists in the database
    */

   public boolean userExists(String myUserID) {
      boolean retVal = false;
      Connection myConnection = null;
      ResultSet myResultSet = null;
      PreparedStatement myStatement = null;

      try {
         if(myUserID != null) {
            myConnection = getConnection();
            myStatement = myConnection.prepareStatement("SELECT user_id " +
                                                        "FROM users " +
                                                        "WHERE user_id = ?");
            myStatement.setString(1, myUserID);
            myResultSet = myStatement.executeQuery();
            if(myResultSet.next()) { // user exists!
               retVal = true;
            }
         }
      }
      catch (SQLException e) {
         logger.fatal("userExists call failed: " + e.getMessage());
	   }
      finally { // clean up
         try {
            if(myStatement != null) {
               myStatement.close();
            }
            if(myResultSet != null) {
               myResultSet.close();
            }
         } catch(SQLException anotherException) {
            logger.fatal(anotherException.getMessage());
         };
         // return connection to pool
         this.myDbBroker.freeConnection(myConnection);
      }
      // return envelope
      return retVal;
   }

   /**
    * getGSMSubscriberID retuns the GSM subscriber ID for a specified user.
    *
    * @param myUserID String containing a user id
    * @return String containing GSM Subscriber ID, or null if the user did not exist
    */

   public String getGSMSubscriberID(String myUserID) {
      String retVal = null;
      Connection myConnection = null;
      ResultSet myResultSet = null;
      PreparedStatement myStatement = null;

      try {
         if(myUserID != null) {
            myConnection = getConnection();
            myStatement = myConnection.prepareStatement("SELECT gsm_subscriber_id " +
                                                        "FROM users " +
                                                        "WHERE user_id = ?");
            myStatement.setString(1, myUserID);
            myResultSet = myStatement.executeQuery();
            if(myResultSet.next()) { // user exists!
               retVal = myResultSet.getString("gsm_subscriber_id");
            }
         }
      }
      catch (SQLException e) {
         logger.fatal("getGSMSubscriberID call failed" + e.getMessage());
	   }
      finally { // clean up
         try {
            if(myStatement != null) {
               myStatement.close();
            }
            if(myResultSet != null) {
               myResultSet.close();
            }
         } catch(SQLException anotherException){
         }
         // return connection to pool
         this.myDbBroker.freeConnection(myConnection);
      }
      // return envelope
      return retVal;
   }

 /**
    * getDescription retuns the descriptiopn for a specified user.
    *
    * @param myUserID String containing a user id
    * @return String containing description, or null if the user did not exist
    */

   public String getDescription(String myUserID) {
      String retVal = null;
      Connection myConnection = null;
      ResultSet myResultSet = null;
      PreparedStatement myStatement = null;

      try {
         if(myUserID != null) {
            myConnection = getConnection();
            myStatement = myConnection.prepareStatement("SELECT user_description " +
                                                        "FROM users " +
                                                        "WHERE user_id = ?");
            myStatement.setString(1, myUserID);
            myResultSet = myStatement.executeQuery();
            if(myResultSet.next()) { // user exists!
               retVal = myResultSet.getString("user_description");
            }
         }
      }
      catch (SQLException e) {
         logger.fatal("getDescription call failed" + e.getMessage());
	   }
      finally { // clean up
         try {
            if(myStatement != null) {
               myStatement.close();
            }
            if(myResultSet != null) {
               myResultSet.close();
            }
         } catch(SQLException anotherException){
         }
         // return connection to pool
         this.myDbBroker.freeConnection(myConnection);
      }
      // return envelope
      return retVal;
   }


   /**
    * getPasswordHash retuns the password hash for a specified user.
    *
    * @param myUserID String containing a user id
    * @return byte[] containing password hash, or null if the user did not exist
    */

   public byte[] getPasswordHash(String myUserID) {
      byte[] retVal = null;
      Connection myConnection = null;
      ResultSet myResultSet = null;
      PreparedStatement myStatement = null;

      try {
         if(myUserID != null) {
            myConnection = getConnection();
            myStatement = myConnection.prepareStatement("SELECT password " +
                                                        "FROM users " +
                                                        "WHERE user_id = ?");
            myStatement.setString(1, myUserID);
            myResultSet = myStatement.executeQuery();
            if(myResultSet.next()) { // user exists!
               Blob myBlob = myResultSet.getBlob("password");
               retVal = myBlob.getBytes(0, (int) myBlob.length());
            }
         }
      }
      catch (SQLException e) {
         logger.error("getPasswordHash call failed: " + e.getMessage());
	   }
      finally { // clean up
         try {
            if(myStatement != null) {
               myStatement.close();
            }
            if(myResultSet != null) {
               myResultSet.close();
            }
         } catch(SQLException anotherException) {
            logger.error(anotherException.getMessage());
         };
         // return connection to pool
         this.myDbBroker.freeConnection(myConnection);
      }
      // return envelope
      return retVal;
   }

  /**
    * updateUserInfo updates GSM subscriber ID and description for a specified user
    *
    * @param myUserID String containing userID of the user
    * @param myDescription String containing a description of the user
    * @param myGSMSubscriberID String containing the GSM subscriber ID of the user
    * @return a boolean value indicating whether the user information was update (true) or not (false)
    */

   public boolean updateUserInfo(String myUserID,
                                 String myDescription,
                                 String myGSMSubscriberID) {
      Connection myConnection = null;
      Statement myStatement = null;
      PreparedStatement myPreparedStatement =  null;
      ResultSet myResultSet = null;
		boolean retVal = false;

      if(myUserID != null && myDescription != null && myGSMSubscriberID != null ) {
         if(this.userExists(myUserID)) {
            try {
               myConnection = getConnection();
               myPreparedStatement= myConnection.prepareStatement("UPDATE users " +
                                                                     "SET user_description = ?, " +
                                                                         "gsm_subscribeR_id = ? " +
                                                                   "WHERE user_id = ?");
               myPreparedStatement.setString(1, myDescription);
               myPreparedStatement.setString(2, myGSMSubscriberID);
               myPreparedStatement.setString(3, myUserID);
               myPreparedStatement.execute();

               // commit the update, and we're done
               myConnection.commit();
               logger.info("Updated user information for user with UserID '" + myUserID + "'");
               retVal = true;
            }
            catch (SQLException e) {
               logger.error("Unable to update user information for user with userID '" + myUserID + "': " + e.getMessage());
            }
            finally { // clean up
               try {
                  if(myStatement != null) {
                     myStatement.close();
                  }
               } catch(SQLException anotherException) {
                  logger.error("Unable to update user information: " + anotherException.getMessage());
               };
               try {
                  if(myPreparedStatement != null) {
                     myPreparedStatement.close();
                  }
               }
               catch(SQLException anotherException) {
                  logger.error("Unable to update user information: " + anotherException.getMessage());
               };
               // return connection to pool
               this.myDbBroker.freeConnection(myConnection);
            }
         }
         else {
            logger.warn("Unable to change user information for non-existent user");
         }
      }
      else {
         logger.warn("Unable to update user information for user with null user id/values");
      }
      return retVal;
   }

   /**
    * getConnection gets a Connection object from the connection pool
    * sets autocommit to false, and returns the object. Invoking
    * object has the responsibility for releasing the connection
    * upon completion. The connection pool must have been initialized
    * successfully for this method to return a valid Connection object
    * instance.
    *
    * @return a Connection object with autocommit set to false
    */

   private Connection getConnection() {
      Connection myConnection = null;

      try {
         myConnection = this.myDbBroker.getConnection();
         myConnection.setAutoCommit(false);
      } catch(SQLException e) {
         logger.fatal("Unable to get database connection: " + e.getMessage());
      }
      return myConnection;
   }
}